@@ -57,10 +57,12 @@
 
     BOOL _swipeGesturesEnabled;
     NSString * panType;
+    UIPinchGestureRecognizer *_pinchRecognizer;
     UIPanGestureRecognizer *_panRecognizer;
     UISwipeGestureRecognizer *_swipeRecognizerLeft;
     UISwipeGestureRecognizer *_swipeRecognizerRight;
     UITapGestureRecognizer *_tapRecognizer;
+    UITapGestureRecognizer *_tapOnVideoRecognizer;
 }
 
 @property (nonatomic, strong) UIPopoverController *masterPopoverController;
@@ -78,6 +80,8 @@
 
 - (void)dealloc
 {
+    [[NSNotificationCenter defaultCenter] removeObserver:self];
+
     if (_tapRecognizer)
         [self.view removeGestureRecognizer:_tapRecognizer];
     if (_swipeRecognizerLeft)
@@ -86,17 +90,27 @@
         [self.view removeGestureRecognizer:_swipeRecognizerRight];
     if (_panRecognizer)
         [self.view removeGestureRecognizer:_panRecognizer];
+    if (_pinchRecognizer)
+        [self.view removeGestureRecognizer:_pinchRecognizer];
+    [self.view removeGestureRecognizer:_tapOnVideoRecognizer];
+
+    _tapRecognizer = nil;
+    _swipeRecognizerLeft = nil;
+    _swipeRecognizerRight = nil;
+    _panRecognizer = nil;
+    _pinchRecognizer = nil;
+    _tapOnVideoRecognizer = nil;
 
     [[NSNotificationCenter defaultCenter] removeObserver:self];
 }
 
 #pragma mark - Managing the media item
 
-- (void)setMediaItem:(id)newMediaItem
+- (void)setFileFromMediaLibrary:(id)newFile
 {
-    if (_mediaItem != newMediaItem) {
+    if (_fileFromMediaLibrary != newFile) {
         [self _stopPlayback];
-        _mediaItem = newMediaItem;
+        _fileFromMediaLibrary = newFile;
         if (_viewAppeared)
             [self _startPlayback];
     }
@@ -204,6 +218,8 @@
                    name:UIApplicationDidBecomeActiveNotification object:nil];
     [center addObserver:self selector:@selector(applicationDidEnterBackground:)
                    name:UIApplicationDidEnterBackgroundNotification object:nil];
+    [center addObserver:self selector:@selector(audioSessionRouteChange:)
+                   name:AVAudioSessionRouteChangeNotification object:nil];
 
     _playingExternallyTitle.text = NSLocalizedString(@"PLAYING_EXTERNALLY_TITLE", @"");
     _playingExternallyDescription.text = NSLocalizedString(@"PLAYING_EXTERNALLY_DESC", @"");
@@ -213,18 +229,19 @@
     self.trackNameLabel.text = self.artistNameLabel.text = self.albumNameLabel.text = @"";
 
     _movieView.userInteractionEnabled = NO;
-    UITapGestureRecognizer *tapOnVideoRecognizer = [[UITapGestureRecognizer alloc] initWithTarget:self action:@selector(toggleControlsVisible)];
-    tapOnVideoRecognizer.delegate = self;
-    [self.view addGestureRecognizer:tapOnVideoRecognizer];
+    _tapOnVideoRecognizer = [[UITapGestureRecognizer alloc] initWithTarget:self action:@selector(toggleControlsVisible)];
+    _tapOnVideoRecognizer.delegate = self;
+    [self.view addGestureRecognizer:_tapOnVideoRecognizer];
 
-    _displayRemainingTime = [[[NSUserDefaults standardUserDefaults] objectForKey:kVLCShowRemainingTime] boolValue];
+    NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
+    _displayRemainingTime = [[defaults objectForKey:kVLCShowRemainingTime] boolValue];
+    _swipeGesturesEnabled = [[defaults objectForKey:kVLCSettingPlaybackGestures] boolValue];
 
-    UIPinchGestureRecognizer *pinchRecognizer = [[UIPinchGestureRecognizer alloc] initWithTarget:self action:@selector(handlePinchGesture:)];
-    pinchRecognizer.delegate = self;
-    [self.view addGestureRecognizer:pinchRecognizer];
-
-    _swipeGesturesEnabled = YES;
     if (_swipeGesturesEnabled) {
+        _pinchRecognizer = [[UIPinchGestureRecognizer alloc] initWithTarget:self action:@selector(handlePinchGesture:)];
+        _pinchRecognizer.delegate = self;
+        [self.view addGestureRecognizer:_pinchRecognizer];
+
         _tapRecognizer = [[UITapGestureRecognizer alloc] initWithTarget:self action:@selector(tapRecognized)];
         [_tapRecognizer setNumberOfTouchesRequired:2];
         _panRecognizer = [[UIPanGestureRecognizer alloc] initWithTarget:self action:@selector(panRecognized:)];
@@ -347,6 +364,8 @@
 {
     [super viewWillAppear:animated];
 
+    _swipeGesturesEnabled = [[[NSUserDefaults standardUserDefaults] objectForKey:kVLCSettingPlaybackGestures] boolValue];
+
     [self.navigationController setNavigationBarHidden:YES animated:YES];
 
     if (!SYSTEM_RUNS_IOS7_OR_LATER) {
@@ -383,29 +402,35 @@
 
     NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
 
-    if (!self.mediaItem && !self.url && !self.mediaList) {
+    if (!self.fileFromMediaLibrary && !self.url && !self.mediaList) {
         [self _stopPlayback];
         return;
     }
 
-    _listPlayer = [[VLCMediaListPlayer alloc] initWithOptions:@[[NSString stringWithFormat:@"--%@=%@", kVLCSettingSubtitlesFont, [defaults objectForKey:kVLCSettingSubtitlesFont]], [NSString stringWithFormat:@"--%@=%@", kVLCSettingSubtitlesFontColor, [defaults objectForKey:kVLCSettingSubtitlesFontColor]], [NSString stringWithFormat:@"--%@=%@", kVLCSettingSubtitlesFontSize, [defaults objectForKey:kVLCSettingSubtitlesFontSize]], [NSString stringWithFormat:@"--%@=%@", kVLCSettingDeinterlace, [defaults objectForKey:kVLCSettingDeinterlace]], [NSString stringWithFormat:@"--%@=%@", kVLCSettingNetworkCaching, [defaults objectForKey:kVLCSettingNetworkCaching]]]];
+    _listPlayer = [[VLCMediaListPlayer alloc] init];
     _mediaPlayer = _listPlayer.mediaPlayer;
     [_mediaPlayer setDelegate:self];
     [_mediaPlayer setDrawable:self.movieView];
+    if ([[defaults objectForKey:kVLCSettingDeinterlace] intValue] != 0)
+        [_mediaPlayer setDeinterlaceFilter:@"blend"];
+    else
+        [_mediaPlayer setDeinterlaceFilter:nil];
     self.trackNameLabel.text = self.artistNameLabel.text = self.albumNameLabel.text = @"";
 
     VLCMedia *media;
-    if (self.mediaItem) {
-        MLFile *item = self.mediaItem;
+    if (self.fileFromMediaLibrary) {
+        MLFile *item = self.fileFromMediaLibrary;
         media = [VLCMedia mediaWithURL:[NSURL URLWithString:item.url]];
-        item.unread = @(NO);
-    } else if (!self.mediaList) {
+    } else if (self.mediaList) {
+        media = [self.mediaList mediaAtIndex:self.itemInMediaListToBePlayedFirst];
+        [media parse];
+    } else {
         media = [VLCMedia mediaWithURL:self.url];
         [media parse];
     }
 
     NSMutableDictionary *mediaDictionary = [[NSMutableDictionary alloc] init];
-
+    [mediaDictionary setObject:[defaults objectForKey:kVLCSettingNetworkCaching] forKey:kVLCSettingNetworkCaching];
     [mediaDictionary setObject:[[defaults objectForKey:kVLCSettingStretchAudio] boolValue] ? kVLCSettingStretchAudioOnValue : kVLCSettingStretchAudioOffValue forKey:kVLCSettingStretchAudio];
     [mediaDictionary setObject:[defaults objectForKey:kVLCSettingTextEncoding] forKey:kVLCSettingTextEncoding];
     [mediaDictionary setObject:[defaults objectForKey:kVLCSettingSkipLoopFilter] forKey:kVLCSettingSkipLoopFilter];
@@ -463,7 +488,7 @@
     [self.repeatButtonLandscape setImage:[UIImage imageNamed:@"repeat"] forState:UIControlStateNormal];
 
     if (![self _isMediaSuitableForDevice]) {
-        UIAlertView * alert = [[UIAlertView alloc] initWithTitle:NSLocalizedString(@"DEVICE_TOOSLOW_TITLE", @"") message:[NSString stringWithFormat:NSLocalizedString(@"DEVICE_TOOSLOW", @""), [[UIDevice currentDevice] model], self.mediaItem.title] delegate:self cancelButtonTitle:NSLocalizedString(@"BUTTON_CANCEL", @"") otherButtonTitles:NSLocalizedString(@"BUTTON_OPEN", @""), nil];
+        UIAlertView * alert = [[UIAlertView alloc] initWithTitle:NSLocalizedString(@"DEVICE_TOOSLOW_TITLE", @"") message:[NSString stringWithFormat:NSLocalizedString(@"DEVICE_TOOSLOW", @""), [[UIDevice currentDevice] model], self.fileFromMediaLibrary.title] delegate:self cancelButtonTitle:NSLocalizedString(@"BUTTON_CANCEL", @"") otherButtonTitles:NSLocalizedString(@"BUTTON_OPEN", @""), nil];
         [alert show];
     } else
         [self _playNewMedia];
@@ -474,10 +499,10 @@
 
 - (BOOL)_isMediaSuitableForDevice
 {
-    if (!self.mediaItem)
+    if (!self.fileFromMediaLibrary)
         return YES;
 
-    NSUInteger totalNumberOfPixels = [[[self.mediaItem videoTrack] valueForKey:@"width"] doubleValue] * [[[self.mediaItem videoTrack] valueForKey:@"height"] doubleValue];
+    NSUInteger totalNumberOfPixels = [[[self.fileFromMediaLibrary videoTrack] valueForKey:@"width"] doubleValue] * [[[self.fileFromMediaLibrary videoTrack] valueForKey:@"height"] doubleValue];
 
     NSInteger speedCategory = [[UIDevice currentDevice] speedCategory];
 
@@ -509,16 +534,26 @@
 {
     NSNumber *playbackPositionInTime = @(0);
     CGFloat lastPosition = .0;
-    NSInteger duration = self.mediaItem.duration.intValue;
+    NSInteger duration = 0;
+    MLFile *matchedFile;
 
-    if (self.mediaItem.lastPosition)
-        lastPosition = self.mediaItem.lastPosition.floatValue;
+    if (self.fileFromMediaLibrary)
+        matchedFile = self.fileFromMediaLibrary;
+    else if (self.mediaList) {
+        NSArray *matches = [MLFile fileForURL:[[[self.mediaList mediaAtIndex:self.itemInMediaListToBePlayedFirst] url] absoluteString]];
+        if (matches.count > 0) {
+            matchedFile = matches[0];
+            lastPosition = matchedFile.lastPosition.floatValue;
+        }
+    }
+    if (matchedFile.lastPosition)
+        lastPosition = matchedFile.lastPosition.floatValue;
+    duration = matchedFile.duration.intValue;
     if (lastPosition < .95) {
         if (duration != 0)
             playbackPositionInTime = @(lastPosition * (duration / 1000.));
     }
     if (playbackPositionInTime.intValue > 0 && (duration * lastPosition - duration) < -60000) {
-        /* start time is not supported for media lists */
         [_mediaPlayer.media addOptions:@{@"start-time": playbackPositionInTime}];
         APLog(@"set starttime to %i", playbackPositionInTime.intValue);
     }
@@ -531,11 +566,11 @@
     else
         [_listPlayer playMedia:_listPlayer.rootMedia];
 
-    if (self.mediaItem) {
-        if (self.mediaItem.lastAudioTrack.intValue > 0)
-            _mediaPlayer.currentAudioTrackIndex = self.mediaItem.lastAudioTrack.intValue;
-        if (self.mediaItem.lastSubtitleTrack.intValue > 0)
-            _mediaPlayer.currentVideoSubTitleIndex = self.mediaItem.lastSubtitleTrack.intValue;
+    if (matchedFile) {
+        if (matchedFile.lastAudioTrack.intValue > 0)
+            _mediaPlayer.currentAudioTrackIndex = matchedFile.lastAudioTrack.intValue;
+        if (matchedFile.lastSubtitleTrack.intValue > 0)
+            _mediaPlayer.currentVideoSubTitleIndex = matchedFile.lastSubtitleTrack.intValue;
     }
 
     self.playbackSpeedSlider.value = [self _playbackSpeed];
@@ -595,8 +630,8 @@
         if (_listPlayer)
             _listPlayer = nil;
     }
-    if (_mediaItem)
-        _mediaItem = nil;
+    if (_fileFromMediaLibrary)
+        _fileFromMediaLibrary = nil;
     if (_mediaList)
         _mediaList = nil;
     if (_url)
@@ -607,9 +642,9 @@
 
 - (void)_saveCurrentState
 {
-    if (self.mediaItem) {
+    if (self.fileFromMediaLibrary) {
         @try {
-            MLFile *item = self.mediaItem;
+            MLFile *item = self.fileFromMediaLibrary;
             item.lastPosition = @([_mediaPlayer position]);
             item.lastAudioTrack = @(_mediaPlayer.currentAudioTrackIndex);
             item.lastSubtitleTrack = @(_mediaPlayer.currentVideoSubTitleIndex);
@@ -617,6 +652,53 @@
         @catch (NSException *exception) {
             APLog(@"failed to save current media state - file removed?");
         }
+    } else {
+        NSArray *files = [MLFile fileForURL:[[_mediaPlayer.media url] absoluteString]];
+        if (files.count > 0) {
+            MLFile *fileFromList = files[0];
+            fileFromList.lastPosition = @([_mediaPlayer position]);
+            fileFromList.lastAudioTrack = @(_mediaPlayer.currentAudioTrackIndex);
+            fileFromList.lastSubtitleTrack = @(_mediaPlayer.currentVideoSubTitleIndex);
+        }
+    }
+}
+
+- (NSString *)_resolveFontName
+{
+    NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
+    BOOL bold = [[defaults objectForKey:kVLCSettingSubtitlesBoldFont] boolValue];
+    NSString *font = [defaults objectForKey:kVLCSettingSubtitlesFont];
+    NSDictionary *fontMap = @{
+                              @"AmericanTypewriter":   @"AmericanTypewriter-Bold",
+                              @"ArialMT":              @"Arial-BoldMT",
+                              @"ArialHebrew":          @"ArialHebrew-Bold",
+                              @"ChalkboardSE-Regular": @"ChalkboardSE-Bold",
+                              @"CourierNewPSMT":       @"CourierNewPS-BoldMT",
+                              @"Georgia":              @"Georgia-Bold",
+                              @"GillSans":             @"GillSans-Bold",
+                              @"GujaratiSangamMN":     @"GujaratiSangamMN-Bold",
+                              @"STHeitiSC-Light":      @"STHeitiSC-Medium",
+                              @"STHeitiTC-Light":      @"STHeitiTC-Medium",
+                              @"HelveticaNeue":        @"HelveticaNeue-Bold",
+                              @"HiraKakuProN-W3":      @"HiraKakuProN-W6",
+                              @"HiraMinProN-W3":       @"HiraMinProN-W6",
+                              @"HoeflerText-Regular":  @"HoeflerText-Black",
+                              @"Kailasa":              @"Kailasa-Bold",
+                              @"KannadaSangamMN":      @"KannadaSangamMN-Bold",
+                              @"MalayalamSangamMN":    @"MalayalamSangamMN-Bold",
+                              @"OriyaSangamMN":        @"OriyaSangamMN-Bold",
+                              @"SinhalaSangamMN":      @"SinhalaSangamMN-Bold",
+                              @"SnellRoundhand":       @"SnellRoundhand-Bold",
+                              @"TamilSangamMN":        @"TamilSangamMN-Bold",
+                              @"TeluguSangamMN":       @"TeluguSangamMN-Bold",
+                              @"TimesNewRomanPSMT":    @"TimesNewRomanPS-BoldMT",
+                              @"Zapfino":              @"Zapfino"
+                              };
+
+    if (!bold) {
+        return font;
+    } else {
+        return fontMap[font];
     }
 }
 
@@ -660,12 +742,15 @@
 
         case UIEventSubtypeRemoteControlNextTrack:
             [self forward:nil];
+            break;
 
         case UIEventSubtypeRemoteControlPreviousTrack:
             [self backward:nil];
+            break;
 
         case UIEventSubtypeRemoteControlStop:
             [self closePlayback:nil];
+            break;
 
         default:
             break;
@@ -676,12 +761,18 @@
 
 - (void)handlePinchGesture:(UIPinchGestureRecognizer *)recognizer
 {
+    if (!_swipeGesturesEnabled)
+        return;
+
     if (recognizer.velocity < 0.)
         [self closePlayback:nil];
 }
 
 - (BOOL)gestureRecognizer:(UIGestureRecognizer *)gestureRecognizer shouldReceiveTouch:(UITouch *)touch
 {
+    if (!_swipeGesturesEnabled)
+        return NO;
+
     if (touch.view != self.view)
         return NO;
 
@@ -690,6 +781,9 @@
 
 - (BOOL)gestureRecognizer:(UIGestureRecognizer *)gestureRecognizer shouldRecognizeSimultaneouslyWithGestureRecognizer:(UIGestureRecognizer *)otherGestureRecognizer
 {
+    if (!_swipeGesturesEnabled)
+        return NO;
+
     return YES;
 }
 
@@ -794,7 +888,7 @@
      * wouldn't see the I-frames when seeking on current mobile devices. This isn't a problem
      * within the Simulator, but especially on older ARMv7 devices, it's clearly noticeable. */
     [self performSelector:@selector(_setPositionForReal) withObject:nil afterDelay:0.3];
-    VLCTime *newPosition = [VLCTime timeWithInt:(int)(_positionSlider.value * self.mediaItem.duration.intValue)];
+    VLCTime *newPosition = [VLCTime timeWithInt:(int)(_positionSlider.value * self.fileFromMediaLibrary.duration.intValue)];
     [self.timeDisplay setTitle:newPosition.stringValue forState:UIControlStateNormal];
     self.timeDisplay.accessibilityLabel = [NSString stringWithFormat:@"%@: %@", NSLocalizedString(@"PLAYBACK_POSITION", @""), newPosition.stringValue];
     _positionSet = NO;
@@ -867,6 +961,11 @@
         _mediaPlayer.media.delegate = self;
         /* let's update meta data */
         [self _updateDisplayedMetadata];
+
+        /* on-the-fly values through private API */
+        [_mediaPlayer performSelector:@selector(setTextRendererFont:) withObject:[self _resolveFontName]];
+        [_mediaPlayer performSelector:@selector(setTextRendererFontSize:) withObject:[[NSUserDefaults standardUserDefaults] objectForKey:kVLCSettingSubtitlesFontSize]];
+        [_mediaPlayer performSelector:@selector(setTextRendererFontColor:) withObject:[[NSUserDefaults standardUserDefaults] objectForKey:kVLCSettingSubtitlesFontColor]];
     }
 
     if (currentState == VLCMediaPlayerStateError) {
@@ -875,7 +974,8 @@
     }
 
     if ((currentState == VLCMediaPlayerStateEnded || currentState == VLCMediaPlayerStateStopped) && _listPlayer.repeatMode == VLCDoNotRepeat) {
-        if ([_listPlayer.mediaList indexOfMedia:_mediaPlayer.media] == _listPlayer.mediaList.count - 1)[self performSelector:@selector(closePlayback:) withObject:nil afterDelay:2.];
+        if ([_listPlayer.mediaList indexOfMedia:_mediaPlayer.media] == _listPlayer.mediaList.count - 1)
+            [self performSelector:@selector(closePlayback:) withObject:nil afterDelay:2.];
     }
 
     UIImage *playPauseImage = [_mediaPlayer isPlaying]? [UIImage imageNamed:@"pauseIcon"] : [UIImage imageNamed:@"playIcon"];
@@ -975,7 +1075,8 @@
     [_subtitleActionSheet showInView:(UIButton *)sender];
 }
 
-- (void)actionSheet:(UIActionSheet *)actionSheet clickedButtonAtIndex:(NSInteger)buttonIndex {
+- (void)actionSheet:(UIActionSheet *)actionSheet clickedButtonAtIndex:(NSInteger)buttonIndex
+{
     if (buttonIndex == [actionSheet cancelButtonIndex])
         return;
 
@@ -1068,10 +1169,14 @@
             musicPlayer.volume += 0.01;
     } else if ([panType isEqual:@"Brightness"]) {
         CGFloat brightness = [UIScreen mainScreen].brightness;
+
         if (panDirectionY > 0)
-            [[UIScreen mainScreen] setBrightness:(brightness - 0.01)];
+            brightness = brightness - 0.01;
         else
-            [[UIScreen mainScreen] setBrightness:(brightness + 0.01)];
+            brightness = brightness + 0.01;
+
+        [[UIScreen mainScreen] setBrightness:brightness];
+        self.brightnessSlider.value = brightness * 2.;
 
         NSString *brightnessHUD = [NSString stringWithFormat:@"%@: %@ %%", NSLocalizedString(@"VFILTER_BRIGHTNESS", @""), [[[NSString stringWithFormat:@"%f",(brightness*100)] componentsSeparatedByString:@"."] objectAtIndex:0]];
         [self.statusLabel showStatusMessage:brightnessHUD];
@@ -1277,6 +1382,15 @@
     }
 }
 
+- (void)audioSessionRouteChange:(NSNotification *)notification
+{
+    NSArray *outputs = [[AVAudioSession sharedInstance] currentRoute].outputs;
+    NSString *portName = [[outputs objectAtIndex:0] portName];
+
+    if (![portName isEqualToString:@"Headphones"])
+        [_listPlayer pause];
+}
+
 - (void)mediaDidFinishParsing:(VLCMedia *)aMedia
 {
     [self _updateDisplayedMetadata];
@@ -1289,11 +1403,18 @@
 
 - (void)_updateDisplayedMetadata
 {
-    MLFile *item = self.mediaItem;
+    MLFile *item;
     NSString *title;
     NSString *artist;
     NSString *albumName;
     NSString *trackNumber;
+    if (self.fileFromMediaLibrary)
+        item = self.fileFromMediaLibrary;
+    else if (self.mediaList) {
+        NSArray *matches = [MLFile fileForURL:[_mediaPlayer.media.url absoluteString]];
+        if (matches.count > 0)
+            item = matches[0];
+    }
 
     if (item) {
         if (item.isAlbumTrack) {
@@ -1360,7 +1481,8 @@
            || toInterfaceOrientation != UIInterfaceOrientationPortraitUpsideDown;
 }
 
-- (void)willRotateToInterfaceOrientation:(UIInterfaceOrientation)toInterfaceOrientation duration:(NSTimeInterval)duration {
+- (void)willRotateToInterfaceOrientation:(UIInterfaceOrientation)toInterfaceOrientation duration:(NSTimeInterval)duration
+{
     [super willRotateToInterfaceOrientation:toInterfaceOrientation duration:duration];
 
     if (UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPhone) {
