@@ -23,6 +23,7 @@
 @property (nonatomic, strong) IBOutlet VLCLinearProgressIndicator *progressView;
 @property (nonatomic, strong) IBOutlet UIView *mediaIsUnreadView;
 @property (nonatomic, strong) IBOutlet UIImageView *isSelectedView;
+@property (nonatomic, strong) IBOutlet UIImageView *folderIconView;
 
 @property (nonatomic, retain) MLFile *mediaObject;
 
@@ -30,5 +31,6 @@
 
 - (void)setEditing:(BOOL)editing animated:(BOOL)animated;
 - (void)selectionUpdate;
+- (void)shake:(BOOL)shake;
 
 @end
